from typing import Optional

from fastapi import APIRouter, Depends, HTTPException, Query, status
from sqlalchemy.orm import Session

from app.crud import todo as todo_crud
from app.db.session import get_db
from app.schemas.todo import Todo, TodoCreate, TodoList, TodoUpdate

router = APIRouter(tags=["todos"])


@router.get("/todos", response_model=TodoList)
async def get_todos(
    skip: int = Query(0, ge=0, description="Number of items to skip"),
    limit: int = Query(100, ge=1, le=100, description="Number of items to return"),
    completed: Optional[bool] = Query(None, description="Filter by completion status"),
    db: Session = Depends(get_db),
):
    """
    Get a list of todo items with pagination and optional filtering by completion status.
    """
    todos = todo_crud.get_todos(db, skip=skip, limit=limit, completed=completed)
    count = todo_crud.get_todos_count(db, completed=completed)
    return {"items": todos, "count": count}


@router.post("/todos", response_model=Todo, status_code=status.HTTP_201_CREATED)
async def create_todo(
    todo: TodoCreate,
    db: Session = Depends(get_db),
):
    """
    Create a new todo item.
    """
    return todo_crud.create_todo(db, todo)


@router.get("/todos/{todo_id}", response_model=Todo)
async def get_todo(
    todo_id: int,
    db: Session = Depends(get_db),
):
    """
    Get a specific todo item by ID.
    """
    db_todo = todo_crud.get_todo(db, todo_id)
    if db_todo is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Todo with ID {todo_id} not found",
        )
    return db_todo


@router.put("/todos/{todo_id}", response_model=Todo)
async def update_todo(
    todo_id: int,
    todo: TodoUpdate,
    db: Session = Depends(get_db),
):
    """
    Update a specific todo item by ID.
    """
    db_todo = todo_crud.update_todo(db, todo_id, todo)
    if db_todo is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Todo with ID {todo_id} not found",
        )
    return db_todo


@router.delete("/todos/{todo_id}", status_code=status.HTTP_204_NO_CONTENT, response_model=None)
async def delete_todo(
    todo_id: int,
    db: Session = Depends(get_db),
):
    """
    Delete a specific todo item by ID.
    """
    result = todo_crud.delete_todo(db, todo_id)
    if result is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Todo with ID {todo_id} not found",
        )
    return None